/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.*;
import eos.models.*;
import eos.models.Baseline2Model.Baseline2Laborer;

/**
 * Uses a demand curve system to dynamically adjust the price of food.
 * 
 * @author Michael Adelson
 */
public class BL2FoodDemandCurveOL extends Baseline2Laborer {

    /**
     * A comfortable supply of food.
     */
    private static final double TARGET_FOOD_STOCK = 30.0;

    /**
     * A comfortable stock of cash.
     */
    private final double TARGET_MONEY_STOCK;

    /**
     * @param economy
     * @param goods
     */
    public BL2FoodDemandCurveOL(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        this.TARGET_MONEY_STOCK = this.money.getQuantity();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline2Model.Baseline2Laborer#baseLine2LaborerSubclassAct()
     */
    protected void baseLine2LaborerSubclassAct() {
        // sell labor statically
        if (this.firms == null || this.firms.isEmpty()) {
            double moneyNeeded = this.TARGET_MONEY_STOCK
                    - this.money.getQuantity();
            if (moneyNeeded > 0.0)
                this.laborMarket.addSellOffer(this, Math.min(moneyNeeded
                        / Baseline2Model.HISTORICAL_LABOR_PRICE,
                        Laborer.LABOR_ALLOWANCE),
                        Baseline2Model.HISTORICAL_LABOR_PRICE);
            else if (this.food.getQuantity() < 0.75 * TARGET_FOOD_STOCK)
                this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE,
                        Baseline2Model.HISTORICAL_LABOR_PRICE);
        } else if (this.money.getQuantity() < this.TARGET_MONEY_STOCK) { // firm
            // owner
            this.firms.get(0).requestTransfer(this.money.getName(),
                    this.TARGET_MONEY_STOCK - this.money.getQuantity());
        }

        // initial buy
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0) {
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    Baseline2Model.HISTORICAL_FOOD_PRICE);
            return;
        }

        double b = this.laborMarket.getLastMarketPrice();
        double m = -(b - this.foodMarket.getLastMarketPrice())
                / TARGET_FOOD_STOCK;

        for (double x = this.food.getQuantity(); x <= 1.1 * TARGET_FOOD_STOCK; x += Laborer.EAT_AMOUNT) {
            double price = m * x + b;
            if (price <= 0.0)
                break;
            // if (this.economy.getRunningTimeSteps() <= 10)
            // System.out.println("#" + this.getID() + " Adding buy offer: "
            // + price + " for x=" + x + " (I have "
            // + this.food.getQuantity() + ")");
            if (!this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT, price))
                break;
        }
    }

}
