/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Baseline2Model;
import eos.models.Laborer;
import eos.models.Baseline2Model.Baseline2Laborer;
import eos.utilities.Errors;

/**
 * The LazyOwner expects to own a Firm. He withdraws enough money from the Firm
 * to buy Food and spends his time at leisure.
 * 
 * @author Michael Adelson
 */
public class BL2LazyOwner extends Baseline2Laborer {

    /**
     * Construct a BL2LazyOwner
     */
    public BL2LazyOwner(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline2Model.Baseline2Laborer#baseLine2LaborerSubclassAct()
     */
    protected void baseLine2LaborerSubclassAct() {
        
        Errors.Assert(this.firms != null && this.firms.size() >= 1,
                "Must own a Firm");

        // ask for enough money to hold money at the same value as
        // FOOD_THRESHOLD
        double moneyNeeded = BL2FixedPriceLaborer.FOOD_THRESHOLD
                - this.money.getQuantity();
        if (moneyNeeded > 0.0)
            this.firms.get(0)
                    .requestTransfer(this.money.getName(), moneyNeeded);

        // buy enough food to stay at or above FOOD_THRESHOLD
        double foodNeeded = BL2FixedPriceLaborer.FOOD_THRESHOLD
                - this.food.getQuantity();
        if (foodNeeded > 0.0)
            this.foodMarket.addBuyOffer(this, foodNeeded,
                    Baseline2Model.HISTORICAL_FOOD_PRICE);
        
        if (foodNeeded > 5.0 * Laborer.EAT_AMOUNT) {
            this.firms.get(0).requestTransfer(this.food.getName(), Laborer.EAT_AMOUNT);
        }
        
        if (this.food.getQuantity() < 5.0)
            System.out.println("PANIC!" + this.toString());
    }
}
