/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import eos.utilities.Errors;
import static eos.models.Baseline3Model.*;

/**
 * An implementation for the Utility Factory in Baseline 3
 * 
 * @author Michael Adelson
 * 
 */
public class BL3Factory extends Baseline3Firm {

    private static final double TARGET_UTILITY_STOCK = 60.0;
    
    /**
     * @see Baseline3Firm#Baseline3Firm(Economy, Map)
     */
    public BL3Factory(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        
        Errors.Assert(!this.isFarm);
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline3Model.Baseline3Firm#baseLine3FirmSubclassAct()
     */
    protected void baseLine3FirmSubclassAct() {
        // sell product (fixed prices)
        double productQuantity = this.product.getQuantity();
        if (this.product.getQuantity() > 0.0)
            this.productMarket.addSellOffer(this, this.product.getQuantity(),
                    getHistoricalPrice(this.product));

        // buy labor
        // initialize the market
        if (this.laborMarket.getLastMarketPrice() <= 0.0
                || this.productMarket.getLastMarketPrice() <= 0.0)
            this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                    getHistoricalPrice(this.labor));
        else { // use a demand curve
            double lastProduct = 0.0, minPrice = (Laborer.LABOR_ALLOWANCE * this.laborMarket
                    .getLastMarketPrice()) / 2.0;
            for (double d = Laborer.LABOR_ALLOWANCE; this.money.getQuantity() > 0.0; d += Laborer.LABOR_ALLOWANCE) {
                double marginalProduct = this.convertToProduct(d) - lastProduct;
                
                if (lastProduct + productQuantity >= TARGET_UTILITY_STOCK)
                    break;

                double price = marginalProduct
                        * this.productMarket.getLastMarketPrice();

                // stop when you'd be buying at < 1/2 the price of labor
                if (price < minPrice)
                    break;

                this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                        price);

                lastProduct += marginalProduct;
            }
        }
    }
}
