/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Laborer;
import static eos.models.Baseline3Model.*;

/**
 * A lazy owner implementation for the baseline 3 economy. 
 * 
 * @author Michael Adelson
 * 
 */
public class BL3Owner extends Baseline3Laborer {

    /**
     * Target values used to assess need and make bidding decisions.
     */
    private static final double TARGET_FOOD_STOCK = 30.0,
            TARGET_MONEY_STOCK = 60.0, TARGET_FIRM_MONEY_STOCK = 100.0;

    /**
     * @see Baseline3Laborer#Baseline3Laborer(Economy, Map)
     */
    public BL3Owner(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        // TODO Auto-generated constructor stub
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline3Model.Baseline3Laborer#baseLine3LaborerSubclassAct()
     */
    protected void baseLine3LaborerSubclassAct() {
        // withdraw money
        final Good firmMoney = this.firms.get(0).asAgent().getGood(
                this.money.getName());
        double moneyToWithdraw = Math.max(TARGET_MONEY_STOCK
                - this.money.getQuantity(), firmMoney.getQuantity()
                - TARGET_FIRM_MONEY_STOCK);
        if (moneyToWithdraw > 0.0)
            this.firms.get(0).requestTransfer(this.money.getName(),
                    moneyToWithdraw);

        // buy food
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0)
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    HISTORICAL_FOOD_PRICE);
        else {
            final double quantum = Laborer.EAT_AMOUNT; // Laborer.EAT_AMOUNT /
                                                       // 10.0;
            double m = (this.foodMarket.getLastMarketPrice() - this.laborMarket
                    .getLastMarketPrice())
                    / TARGET_FOOD_STOCK;
            double b = this.laborMarket.getLastMarketPrice();

            if (m > 0.0)
                this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                        this.foodMarket.getLastMarketPrice());

            for (double x = this.food.getQuantity(); x <= 1.5 * TARGET_FOOD_STOCK; x += quantum) {
                double price = m * x + b;
                if (price <= 0.0)
                    break;
                this.foodMarket.addBuyOffer(this, quantum, price);
            }
        }

        // buy utility
        double utilityToBuy = (this.money.getQuantity() - TARGET_MONEY_STOCK)
                / getHistoricalPrice(this.utility);
        if (utilityToBuy > 0.0)
            this.utilityMarket.addBuyOffer(this, utilityToBuy,
                    getHistoricalPrice(this.utility));
    }

}
