/**
 * 
 */
package eos.agents;

import java.util.*;

import eos.framework.*;
import eos.models.BaselineModel.*;
import eos.utilities.Errors;

/**
 * @author Michael Adelson
 * 
 */
public class FixedPriceFarm extends BaselineFarm {

    private Market foodMarket, laborMarket;

    /**
     * Construct a FixedPriceFarm. Markets must be added to economy before
     * calling this constructor.
     */
    public FixedPriceFarm(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        foodMarket = this.economy.getMarketFor(this.money.getName(), this.food
                .getName());
        laborMarket = this.economy.getMarketFor(this.money.getName(),
                this.labor.getName());

        Errors.Assert(foodMarket != null && laborMarket != null,
                "Markets must be added to economy BEFORE FixedPriceLaborers");
    }

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Farm#bid()
     */
    protected void bid() {
        // buy labor
        double laborQuantity = this.money.getQuantity()
                / FixedPriceLaborer.LABOR_PRICE;
        laborQuantity = Math.min(laborQuantity, 60 / 2.5);
        if (laborQuantity > 0.0)
            this.laborMarket.addBuyOffer(this, laborQuantity,
                    FixedPriceLaborer.LABOR_PRICE);

        // sell food
        if (this.food.getQuantity() > 0.0)
            this.foodMarket.addSellOffer(this, this.food.getQuantity(),
                    FixedPriceLaborer.FOOD_PRICE);
    }

}
