package eos.io;

import eos.utilities.Errors;

/**
 * A Printer class which prints in CSV format. Expects to be used with a single
 * Economy object.
 * 
 * by Michael Adelson
 */
public class CSVPrinter extends Printer {

    /**
     * Comma separator.
     */
    protected static final String CS = ",";

    /**
     * Calls the base constructor.
     */
    public CSVPrinter(String fileName, int period) {
        super(fileName, period);
    }

    /**
     * Closes the PrintWriter. After printing DataPrinter cleanup data.
     */
    public void cleanup(DataOutputEconomy economy) {
        StringBuilder sb = new StringBuilder();
        for (DataPrinter dataPrinter : dataPrinters) {
            sb.append(dataPrinter.getCleanupData(economy));
            sb.append(CS);
        }

        printWriter.println(sb.toString());
        printWriter.close();
    }

    /**
     * Prints the data from each DataPrinter, separating values with commas.
     */
    public void print(DataOutputEconomy economy) {
        if ((economy.getRunningTimeSteps() % this.period) != 0)
            return;

        StringBuilder sb = new StringBuilder();
        for (DataPrinter dataPrinter : dataPrinters) {
            sb.append(dataPrinter.getData(economy));
            sb.append(CS);
        }

        printWriter.println(sb.toString());
    }

    /**
     * Prints the title from each DataPrinter, separating values with commas.
     */
    public void printTitles(DataOutputEconomy economy) {
        StringBuilder sb = new StringBuilder();
        for (DataPrinter dataPrinter : dataPrinters) {
            sb.append(dataPrinter.getTitle(economy));
            sb.append(CS);
        }

        printWriter.println(sb.toString());
    }

    public void addDataPrinter(DataPrinter dataPrinter) {
        if (!dataPrinter.isCSVCompatible())
            Errors.err("dataPrinter + " + dataPrinter
                    + " is not CSV compatible!");
        super.addDataPrinter(dataPrinter);
    }

}
