/**
 * 
 */
package eos.simulations;

import eos.agents.*;
import eos.economies.BL321Economy;
import eos.framework.Economy;
import eos.io.Simulation;
import eos.io.Simulation.InitializationFunction;
import eos.markets.CallAuctionMarket;
import eos.models.Baseline2Model;
import eos.models.Baseline2Model.Baseline2Good;

/**
 * @author madelson
 *
 */
public class DynamicSim {

    // parameters
    static int numFarms = 10;
    static int numLaborers = 45 * numFarms;
    
    static Simulation sim;
    
    static {
        //sim = new Simulation(F09Economy.class, new Random().nextLong());
        sim = new Simulation("Dynamic", BL321Economy.class, 5089041334601291732L);
        sim.addGoods(Baseline2Good.class, "Food", "Labor", "Money", "Leisure");
        
        sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Labor");
        sim.addAgents(BL2DynamicOL.class, numLaborers);
        sim.addOwnersAndFirms(BL2DynamicOL.class, numFarms, BL2DynamicFarm.class, 1);
        
        sim.setQuantities(BL2DynamicOL.class, 35, 0, 60, 0);
        sim.setQuantities(BL2DynamicFarm.class, 20, 0, 100, 0);
        
        // set the prices before running
        sim.addInitializationFunction(new InitializationFunction() {
            public void initialize(Economy economy) {
                Baseline2Model.HISTORICAL_LABOR_PRICE = (59.0 / 24.0)
                * Baseline2Model.HISTORICAL_FOOD_PRICE;
            }
        });
    }
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        alphaTest();
    }
    
    public static void alphaTest() {
        sim.setDefaultPrintPeriod(1);
        sim.setPerturbation(10);
        sim.disableDefaultCSVPrinter();
        int numToAdd = 4;
        sim.addAgents(BL2DynamicOL.class, numToAdd);
        sim.enableDefaultMultiRunPrinter("Dynamic.csv", 10);
        sim.run(5000);
        sim.addAgents(BL2DynamicOL.class, (5 * numFarms) - numToAdd);
        sim.run(2000);
        
        //sim.addDataPrinter(new GoodTrackerDataPrinter("Money", false));
//        for (double d = 0.05; d <= 0.5; d += 0.05) {
//            BL2DynamicFarm.ALPHA = d;
//            sim.run(2000);
//        }
//        
        sim.cleanupPrinters();
    }

}
