package eos.simulations;

import eos.agents.BL2FixedPriceFarm;
import eos.agents.BL2FixedPriceLaborer;
import eos.agents.BL2LazyOwner;
import eos.economies.BL321Economy;
import eos.framework.Economy;
import eos.io.Simulation;
import eos.io.Simulation.InitializationFunction;
import eos.markets.CallAuctionMarket;
import eos.models.Baseline2Model;
import eos.models.Baseline2Model.Baseline2Good;

public class FPSim {
    
    
    // parameters
    static int numLaborers = 60;
    static int numFarms = 1;
    
    static Simulation sim;
    
    static {
        sim = new Simulation("FP", BL321Economy.class, Double.doubleToLongBits(Math.random()));
        sim.addGoods(Baseline2Good.class, "Food", "Labor", "Money", "Leisure");
        
        sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Labor");
        sim.addAgents(BL2FixedPriceLaborer.class, numLaborers);
        //sim.addOwnersAndFirms(BL2LazyOwner.class, numFarms, BL2FixedPriceFarm.class, 1);
        sim.addAgents(BL2FixedPriceFarm.class, numFarms);
        
        sim.createIdentifier("people", BL2FixedPriceLaborer.class, BL2LazyOwner.class);
        sim.setQuantities("people", 30, 0, 30, 0);
        sim.setQuantities(BL2FixedPriceFarm.class, 20, 0, 100, 0);
        
        // set the prices before running
        sim.addInitializationFunction(new InitializationFunction() {
            public void initialize(Economy economy) {
                Baseline2Model.HISTORICAL_LABOR_PRICE = (2.6)
                * Baseline2Model.HISTORICAL_FOOD_PRICE;
            }
        });
    }

    
    /**
     * @param args
     */
    public static void main(String[] args) {
        //sim.disableDefaultCSVPrinter();
        //sim.enableDefaultCleanupPrinter("cleanup.csv");
        //sim.run(5000, 50);
        //sim.setDefaultFileName("fp_sim_out%s.%s");
        sim.setDefaultPrintPeriod(1);
        sim.run(5000);
        //sim.cleanupPrinters();
    }

}
